local base = _G

module('mul_create_server')

local require       = base.require
local pairs         = base.pairs
local table         = base.table
local math          = base.math
local loadfile      = base.loadfile
local setfenv       = base.setfenv
local string        = base.string
local assert        = base.assert
local io            = base.io
local loadstring    = base.loadstring
local print         = base.print
local os            = base.os

local i18n 				= require('i18n')
local Window 			= require('Window')
local U                 = require('me_utilities')
local MsgWindow         = require('MsgWindow')
local DialogLoader      = require('DialogLoader')
local net               = require('net')
local Static            = require('Static')
local Skin              = require('Skin')
local textutil          = require('textutil')
local lfs 				= require('lfs')
local Tools 			= require('tools')
local Button			= require('Button')
local Gui               = require('dxgui')
local server_list       = require('mul_server_list')
local FileDialogFilters	= require('FileDialogFilters')
local MeSettings	 	= require('MeSettings')
local FileDialog 		= require('FileDialog')
local ListBoxItem       = require('ListBoxItem')
local DcsWeb            = require('DcsWeb')
local Advanced          = require('mul_advanced')
local waitScreen        = require('me_wait_screen')
local Chat		        = require('mul_chat')
local music             = require('me_music')
local FileDialogUtils	= require('FileDialogUtils')
local nickname          = require('mul_nickname')

i18n.setup(_M)

cdata = 
{
    multiplayer         = _("MULTIPLAYER"),
    ServerSettings      = _("SERVER SETTINGS"),
    ServerName          = _("Server name"),
    password            = _("Password"),
    playersUpTo         = _("Player limit"),
    ip                  = _("Public IP"),
    port                = _("Port"),
    ServerDesc          = _("Server Description"),
    publicServer        = _("Public server"),
    MissionList         = _("Mission list"),
    save_as             = _("SAVE AS"),
    Open                = _("OPEN"),
    Back                = _("BACK"),
    Start               = _("START"),
    
    addMission          = _('Choose mission file to add'), 
    openListMissions    = _('Choose mission list file to open'),    
    saveListMissions    = _('Save mission list file'),  

    welcome             = _('Welcome to our DCS server!'),   
    Advanced            = _('Advanced'),
    
    tool_Shuffle        = _('Shuffle'),
    tool_Loop           = _('Loop '),
    Player              = _("Player"),
    IntegrityCheck      = _("Integrity Check"),
    
}

local settingsServer = {}

local defaultSettingsServer = {}   
local version = 1
    

--------------------------------------------------
-- 
function create()
    w, h = Gui.GetWindowSize()
    window = DialogLoader.spawnDialogFromFile(base.dialogsDir .. "mul_create_server.dlg", cdata)    
    local container = window.main_panel
 --   container:setBounds(parent:getBounds())
    parent = parent
 
    local main_panel = window.main_panel
    
    pWork                   = main_panel.pWork
    pCenter                 = pWork.pCenter
    pNoVisible              = pCenter.pNoVisible 
    pBtn                    = main_panel.pBtn 
    pUp                     = main_panel.pUp
    pLeftUp                 = pCenter.pLeftUp
    pLeftDown               = pCenter.pLeftDown 
    pRight                  = pCenter.pRight
    pRDBtn                  = pRight.pRDBtn
    pRTbtn                  = pRight.pRTbtn
    lbMissions              = pRight.lbMissions
    eServerDesc             = pLeftDown.eServerDesc    
    
    tbShuffle               = pRight.pRTbtn.tbShuffle
    tbLoop                  = pRight.pRTbtn.tbLoop
    
    btnOpen                 = pRDBtn.bOpen
    btnSaveAs               = pRDBtn.bSaveAs
    btnDel                  = pRDBtn.bDel
    btnAdd                  = pRDBtn.bAdd
    btnDown                 = pRDBtn.bDown
    btnUp                   = pRDBtn.bUp
  
    btnBack                 = pBtn.btnBack
    btnStart                = pBtn.btnStart
    sLoginNickname          = pCenter.sLoginNickname
    ePort                   = pLeftUp.ePort
    eIP                     = pLeftUp.eIP
    spPlayers               = pLeftUp.spPlayers
    eServerName             = pLeftUp.eServerName
    cbPublicServer          = pLeftUp.cbPublicServer
    ePassword               = pLeftUp.ePassword
    btnAdvanced             = pLeftUp.btnAdvanced  
    cbIntegrityCheck        = pLeftUp.cbIntegrityCheck
   
    resize(w, h)
    setupCallbacks()
    
    defaultSettingsServer = net.get_default_server_settings()
    
    --base.U.traverseTable(defaultSettingsServer)
    --base.print("-----defaultSettingsServer----")
  
    loadSettings()

    return window
end

function resize(w, h)
    window:setBounds(0, 0, w, h)
    window.main_panel:setBounds(0, 0, w, h)
    
    local panelW = 1280
    
    pWork:setBounds((w-1280)/2, (h-768)/2 + 46, panelW, 676)
    
 --   pCenter:setBounds(0, 0, panelW, h - 44 - 59)

    pUp:setBounds((w-1280)/2, (h-768)/2, 1280, 36)
    pBtn:setBounds((w-1280)/2, (h-768)/2+768-36, 1280, 36)
    
 --   sLoginNickname:setBounds(w-420, 11, 400, 30)
    
 --   pLeftDown:setBounds(0, 380, 691, h-483)
--    eServerDesc:setBounds(20, 56, 651, h-566)

 --   pRight:setBounds(692, 41, 589, h-144)
 --   pRDBtn:setBounds(0, h-58, 589, 82)
  --  lbMissions:setBounds(30, 48, 520, h-250)
    
end

function loadSettingsRaw()
    local tbl = Tools.safeDoFile(lfs.writedir() .. 'Config/serverSettings.lua', false)
    if (tbl and tbl.cfg and tbl.cfg.version and tbl.cfg.version > 0) then
        settingsServer = tbl.cfg
    else
        settingsServer = defaultSettingsServer
    end 
    return settingsServer
end


function loadSettings()
    loadSettingsRaw()
    settingsServer.advanced.disable_events  = nil --���������
    updatePanel()
end

function saveSettings()
    settingsServer.version = version
    U.saveInFile(settingsServer, 'cfg', lfs.writedir() .. 'Config/serverSettings.lua')	
end

function updateSettingsServer()
    settingsServer.listLoop = tbLoop:getState()
    settingsServer.listShuffle = tbShuffle:getState() 
    
    settingsServer.maxPlayers           = spPlayers:getValue()
    settingsServer.name                 = eServerName:getText()
    settingsServer.isPublic             = cbPublicServer:getState()
    settingsServer.require_pure_clients = cbIntegrityCheck:getState()
    settingsServer.description          = eServerDesc:getText()
    settingsServer.password             = ePassword:getText()
    settingsServer.port                 = ePort:getText()
end

local function addMission(a_fullFileName)
    local filename = U.extractFileName(a_fullFileName)
    local newItem = ListBoxItem.new(filename)
    newItem.filename = filename
    newItem.fullFileName = a_fullFileName
    lbMissions:insertItem(newItem)
end

function updatePanel()
--base.U.traverseTable(settingsServer)
    tbShuffle:setState(settingsServer.listShuffle)
    tbLoop:setState(settingsServer.listLoop) 
    
    spPlayers:setValue(settingsServer.maxPlayers)
    eServerName:setText(settingsServer.name)
    cbPublicServer:setState(settingsServer.isPublic)
    cbIntegrityCheck:setState(settingsServer.require_pure_clients)
    eServerDesc:setText(settingsServer.description)
    ePassword:setText(settingsServer.password)
    
    lbMissions:clear()
    if (settingsServer.missionList) then
        for i = 1, #settingsServer.missionList do
            if FileDialogUtils.getFileExists(settingsServer.missionList[i]) == true then
                addMission(settingsServer.missionList[i])
            end   
        end
    end 
end

function setupCallbacks()
    btnBack.onChange        = onChange_btnBack
    btnOpen.onChange        = onChange_btnOpen
    btnSaveAs.onChange      = onChange_btnSaveAs
    btnAdd.onChange         = onChange_btnAdd
    btnDel.onChange         = onChange_btnDel
    btnDown.onChange        = onChange_btnDown
    btnUp.onChange          = onChange_btnUp
    btnStart.onChange       = onChange_btnStart
    btnAdvanced.onChange    = onChange_btnAdvanced
end 

function onChange_btnAdvanced()
    Advanced.setSettings(settingsServer)
    Advanced.show(true)
end

function onChange_btnStart()

    updateSettingsServer()
    tbl = settingsServer
    tbl.missionList = {}
    
    local selectItem = lbMissions:getSelectedItem()
    local index = 1
    if selectItem ~= nil then
        tbl.missionList[1] = selectItem.fullFileName
        index = 2
    end
  
    for i=0,lbMissions:getItemCount()-1 do
        local item = lbMissions:getItem(i)
        if not (selectItem and selectItem == item) then
            tbl.missionList[index] = item.fullFileName
            index = index + 1   
        end   
        --base.print("---missionList---",i,tbl.missionList[i+1])
    end
    
	--DAWS Addition
	print("DAWS loading... " .. tbl.missionList[1])
	local logpath = lfs.writedir() .. "Logs/mixpath.txt" --lfs.currentdir() .. "AMVI/SaveMission/mixpath.txt"		
	local i, erri = io.open(logpath, "w")
	i:write(tbl.missionList[1])
	i:close()	
	--END DAWS Addition		
	
    saveSettings()
   
    --base.U.traverseTable(tbl)
    --base.print("--settingsServer-start---")
	show(false)
	
    waitScreen.setUpdateFunction(function()
		net.set_name(playerName)    
		net.start_server(tbl)
		music.stop()
			  
		base.mmw.show(false)        
		if base.MapWindow.getVisible() == true then
			base.MapWindow.show(false)
		end 
    end)
end

function onChange_btnBack()
    server_list.show(true)
    show(false)
end

function onChange_btnOpen()
    local path = MeSettings.getListMissionsPath()
    
    local filters = {FileDialogFilters.listMissions(), FileDialogFilters.listMissions()}
    local fullFileName = FileDialog.open(path, filters, cdata.openListMissions)

    if fullFileName then
        lbMissions:clear()
        local tbl = Tools.safeDoFile(fullFileName)
        if (tbl and tbl.missions) then            
            for i = 1, #tbl.missions do
                if FileDialogUtils.getFileExists(tbl.missions[i]) == true then
                    addMission(tbl.missions[i])
                end
            end    
        end  
        MeSettings.setListMissionsPath(fullFileName)        
    end 
    updateStart()
end

function onChange_btnSaveAs()
    local path = MeSettings.getListMissionsPath()
	local filters = {FileDialogFilters.listMissions()}
    local fullFileName = FileDialog.save(path, filters, cdata.saveListMissions)
    
    if fullFileName then
        local listMissions = {}
        for i = 0, lbMissions:getItemCount()-1 do
            listMissions[i+1] = lbMissions:getItem(i).fullFileName
        end
        U.saveInFile(listMissions, 'missions', fullFileName)
        MeSettings.setListMissionsPath(fullFileName)
    end
end

function onChange_btnDel()
    local item = lbMissions:getSelectedItem()
    
    if item then
        local index = lbMissions:getItemIndex(item)
        lbMissions:removeItem(item)
        local nextItem = lbMissions:getItem(index)
        if nextItem then
            lbMissions:selectItem(nextItem)
        end
    end
    updateStart()
end

function onChange_btnAdd()
    local path = MeSettings.getMissionPath()
    local filters = {FileDialogFilters.mission(), FileDialogFilters.mission()}
    local fullFileName = FileDialog.open(path, filters, cdata.addMission)

    if fullFileName then        
        addMission(fullFileName)
        MeSettings.setMissionPath(fullFileName)
    end
    updateStart()
end

local function copyItem(item)
    local newItem = ListBoxItem.new(item.filename)
    newItem.filename = item.filename
    newItem.fullFileName = item.fullFileName
    
    return newItem
end

function onChange_btnDown()
    local item = lbMissions:getSelectedItem()
    --base.print("---item=",item)
    if item then
        local index = lbMissions:getItemIndex(item)
        --base.print("---index=",index,lbMissions:getItemCount())
        if (index + 1) < lbMissions:getItemCount() then
            local newItem = copyItem(item)
            lbMissions:insertItem(newItem,index+2)
            lbMissions:removeItem(item)  
            lbMissions:selectItem(newItem)    
        end
        
    end
end

function onChange_btnUp()
    local item = lbMissions:getSelectedItem()

    if item then
        local index = lbMissions:getItemIndex(item)
        if index > 0 then
            local newItem = copyItem(item)
            lbMissions:insertItem(newItem,index-1)
            lbMissions:removeItem(item) 
            lbMissions:selectItem(newItem)              
        end
    end
end

function show(b)
    if window == nil then
        create()
    end

    if b then                
        if base.__FINAL_VERSION__ == false then
            loadSettings()
        end
        
        updateIP()
        updatePlayer()
        updateStart()
    end   

    window:setVisible(b)    
end

function updatePlayer()
    playerName = nickname:getNickname()
    if playerName == nil then
        sLoginNickname:setText(cdata.Player) 
    else
        sLoginNickname:setText(playerName)  
    end
end

function updateStart()
    if lbMissions:getItemCount() > 0 then
        btnStart:setEnabled(true)
    else
        btnStart:setEnabled(false)
    end
end

function updateIP()
    eIP:setText(DcsWeb.get_data('dcs:whatsmyip'))
    ePort:setText(settingsServer.port)
end

----------------------------------------------------------------------------------
function forceServer()
	net.set_name("<SERVER>")    
	net.start_server(loadSettingsRaw())
end
----------------------------------------------------------------------------------